// beosmain.cxx	BeOS stuff - part of LinCity
// Copyright (c) I J Peters 1995,1996.  Please read the file 'COPYRIGHT'.

// Modified by T.Murai
/////////////////////////////////////////////////////////////////////////
// This source cde is modified from
// BeOS port of xgalaga-2.0 by Mike Steed 
/////////////////////////////////////////////////////////////////////////

#include <Alert.h>
#include <Entry.h>
#include <Path.h>
#include <string.h>
#include <malloc.h>
#include <stdio.h> // debug
#include "lin-city.h"
#include "beosmain.h"

#define kMemPoolSize	16
#define	kBlockSize	sizeof(W_Event)


LinCityView::LinCityView(BRect frame):
    BView(frame, "view", B_FOLLOW_ALL, B_WILL_DRAW),
    _eventLock(), _eventList(), _memPool(kMemPoolSize, kBlockSize, B_OBJECT_CACHE)
{
	SetViewColor(0,0,0);
}


bool
LinCityView::eventsPending()
{
    bool pending;

    _eventLock.Lock();
    pending = !_eventList.IsEmpty();
    _eventLock.Unlock();

    return pending;
}


void
LinCityView::nextEvent(W_Event *event)
{
    _eventLock.Lock();

    if (!_eventList.IsEmpty()) {
	void *p = (void *)_eventList.FirstItem();
	memcpy(event, p, sizeof(W_Event));
	_eventList.RemoveItem(p);
	_memPool.Save(p, kBlockSize);
    }

    _eventLock.Unlock();
}


void
LinCityView::Draw(BRect r)
{
	extern W_Window BaseWin;
	if (!BaseWin)	return;
	BBitmap *m = (BBitmap *)BaseWin->bitmap;
	this->DrawBitmap(m, r, r);
}


void
LinCityView::KeyDown(const char *bytes, int32 numBytes)
{
    if (numBytes == 1) {
	W_Event *event = (W_Event *)_memPool.Get(kBlockSize);
	event->type = W_EV_KEY;
	event->x = event->y = 0;
	int key =  bytes[0];
	switch (key)
	{
		case B_LEFT_ARROW:
			key = 1;
		break;
		
		case B_DOWN_ARROW:
			key = 2;
		break;
		
		case B_UP_ARROW:
			key = 3;
		break;
		
		case B_RIGHT_ARROW:
			key = 4;
		break;
	}
		
	event->key = key;

	_eventLock.Lock();
	_eventList.AddItem((void*)event);
	_eventLock.Unlock();
    }
}


void
LinCityView::KeyUp(const char *bytes, int32 numBytes)
{
    if (numBytes == 1) {
	W_Event *event = (W_Event *)_memPool.Get(kBlockSize);
	event->type = W_EV_KEY_OFF;
	event->x = event->y = 0;
	_eventLock.Lock();
	_eventList.AddItem((void*)event);
	_eventLock.Unlock();
    }
}


void
LinCityView::MouseDown(BPoint p)
{
	int32 buttons = 0;  
	
	Window()->CurrentMessage()->FindInt32( "buttons", &buttons ); 

    W_Event *event = (W_Event *)_memPool.Get(kBlockSize);


    event->type = W_EV_BUTTON;
    event->x = (int)p.x;
    event->y = (int)p.y;
    event->key = 0;
    event->button = buttons;

    _eventLock.Lock();
    _eventList.AddItem((void*)event);
    _eventLock.Unlock();
}
void
LinCityView::MouseUp(BPoint p)
{
    W_Event *event = (W_Event *)_memPool.Get(kBlockSize);
    event->type = W_EV_BUTTON_OFF;
    event->x = (int)p.x;
    event->y = (int)p.y;
    event->key = 0;

    _eventLock.Lock();
    _eventList.AddItem((void*)event);
    _eventLock.Unlock();
}


LinCityWin::LinCityWin():
    BWindow(BRect(50,50,60,60), "LinCity", B_TITLED_WINDOW,
	B_NOT_RESIZABLE)
{
    extern W_Window BaseWin;

    ResizeTo(WINWIDTH + BORDERX * 2, WINHEIGHT + BORDERY * 2);
    LinCityView *view = new LinCityView(Bounds());
    AddChild(view);
    view->MakeFocus();
    Show();

    BBitmap *bitmap = new BBitmap(Bounds(), B_CMAP8, true);
    memset(bitmap->Bits(), 0, bitmap->BitsLength());
    BView *buffer = new BView(Bounds(), "buffer", B_FOLLOW_ALL, B_WILL_DRAW);
    bitmap->AddChild(buffer);

    BaseWin = new struct _W_Window;
    BaseWin->view = (char *)view;
    BaseWin->buffer = (char *)buffer;
    BaseWin->bitmap = (char *)bitmap;
    _zoomed = false;
}


LinCityWin::~LinCityWin()
{
    extern W_Window BaseWin;

    BBitmap *b = (BBitmap *)BaseWin->bitmap;
    delete b;
}


bool
LinCityWin::QuitRequested()
{
    extern int real_quit_flag;
    static bool called = false;

    if (real_quit_flag) {
	called = true;
    }
    else {
	real_quit_flag = 1;
	snooze(100000); // give main thread plenty of time to exit
    }

    if (!called) {
	// The first time, don't destroy the window.  This allows Alt-W to
	// work without crashing the main thread.
	called = true;
	return false;
    }
    else {
	return true;
    }
}


void
LinCityWin::Zoom(BPoint rec_position, float rec_width, float rec_height)
{
    BPoint size(Bounds().Width(), Bounds().Height());
    if (_zoomed)
	size.x /= 2.0, size.y /= 2.0;
    else
	size.x *= 2.0, size.y *= 2.0;

    _zoomed = !_zoomed;

    BWindow::Zoom(Frame().LeftTop(), size.x, size.y);
    BView* view = ChildAt(0);
    view->SetScale(_zoomed ? 2.0 : 1.0);
}


LinCity::LinCity():
    BApplication("application/x-vnd.Suka-LinCity"),
    _mainThread(NULL)
{
    extern char *app_dir;

    app_info info;
    GetAppInfo(&info);
    BEntry entry(&info.ref);
    entry.GetParent(&entry);

    BPath path;
    entry.GetPath(&path);
    app_dir = strdup(path.Path());
}


LinCity::~LinCity()
{
    extern char *app_dir;

    _mainThread->requestQuit();

    free(app_dir);
}


void
LinCity::ArgvReceived(int32 argc, char **argv)
{
	parse_args (argc, argv);
}


void
LinCity::ReadyToRun()
{
    new LinCityWin();
    _mainThread = new MainThread();
    BMessage msg(kMsgGo);
    _mainThread->PostMessage(&msg);
}


void
MainThread::requestQuit()
{
    Lock();
    Quit();
}


void
MainThread::MessageReceived(BMessage *m)
{
 extern int original_main();
 
   switch (m->what) {
	case kMsgGo:
	{
	    original_main();

	    be_app->PostMessage(B_QUIT_REQUESTED);
	    break;
	}
	default:
	    BLooper::MessageReceived(m);
	    break;
    }
}


int
main()
{	
    new LinCity();
    be_app->Run();
    delete be_app;

    return 0;
}

