// lcbeos.cxx  BeOS stuff - part of lin-city
// Copyright (c) I J Peters 1995,1996.  Please read the file 'COPYRIGHT'.

// Modified for BeOS by T.Murai

/////////////////////////////////////////////////////////////////////////
// Portions of this source code is modified from
// BeOS port of xgalaga-2.0 by Mike Steed 
/////////////////////////////////////////////////////////////////////////

#include <stdio.h>
#include <stdlib.h>
#include <unistd.h>
#include <math.h>
#include <float.h>
#include <string.h>
#include <Bitmap.h>
#include <Screen.h>
#include <InterfaceDefs.h> // for key code
#include <TranslationUtils.h> // load_start_image()


#include "lctypes.h"
#include "beosmain.h"
#include "lin-city.h"
#include "lcbeos.h"

//#define USE_IMAGES 1

rgb_color pal[256]; 
W_Window BaseWin;

#define RECT(a,b,c,d) (BRect((a), (b), (a)+(c), (b)+(d)))

///////////////////////////////////////////////////////////////////////////////////////////////
// Palette handling.
///////////////////////////////////////////////////////////////////////////////////////////////
void
setcustompalette ()
{
  char s[100];
  int n, r, g, b, flag[256];
  FILE *inf;
  
  memset (flag, 0, sizeof(flag));
  
  if ((inf = fopen (colour_pal_file, "r")) == 0)
    HandleError ("Can't find the colour pallet file", FATAL);

  while (feof (inf) == 0)
    {
      fgets (s, 99, inf);
      if (sscanf (s, "%u %u %u %u", &n, &r, &g, &b) == 4)
      {
    r = ((r * (1 - gamma_correct_red))
         + (64 * sin ((float) r * M_PI / 128))
         * gamma_correct_red);
    g = ((g * (1 - gamma_correct_green))
         + (64 * sin ((float) g * M_PI / 128))
         * gamma_correct_green);
    b = ((b * (1 - gamma_correct_blue))
         + (64 * sin ((float) b * M_PI / 128))
         * gamma_correct_blue);
    gl_setpalettecolor (n, r, g, b);
    flag[n] = 1;
   }
    }
  fclose (inf);
  for (int i = 0; i < 256; i++)
    {
      if (flag[i] == 0)
      {
    printf ("Colour %d not loaded\n", i);
    HandleError ("Can't continue", FATAL);
      }
    }
}

void
gl_setpalettecolor(int x, int r, int g, int b)
{
  if (x < 0 || x > 255) return;
  
  pal[x].red = ( r * 255) / 60;
  if ( pal[x].red > 255)  pal[x].red = 255;
  pal[x].green = ( g * 255) / 60;
  if (pal[x].green > 255)  pal[x].green = 255;
  pal[x].blue = ( b * 255) / 60;
  if ( pal[x].blue > 255)  pal[x].blue = 255;
}
///////////////////////////////////////////////////////////////////////////////////////////////
// Command line option handling.
///////////////////////////////////////////////////////////////////////////////////////////////
void
parse_args (int argc, char **argv)
{
  int option;
  extern char *optarg;


#ifdef ALLOW_PIX_DOUBLING
  printf ("Options include Pix doubling\n");
  while ((option = getopt (argc, argv, "vbndg:wR:G:B:")) != EOF)
#else
  while ((option = getopt (argc, argv, "vbng:wR:G:B:")) != EOF)
#endif
    {
      switch (option)
  {
  case 'v':
    verbose = TRUE;
    break;
#ifdef ALLOW_PIX_DOUBLING
  case 'd':
    pix_double = 1;
// and let it run through; we are not allowed a border with pix doubling
#endif
  case 'b':
    borderx = 0;
    bordery = 0;
    break;
  case 'n':
    no_init_help = TRUE;
    break;
  case 'w':
    gamma_correct_red = GAMMA_CORRECT_RED;
    gamma_correct_green = GAMMA_CORRECT_GREEN;
    gamma_correct_blue = GAMMA_CORRECT_BLUE;
    break;
  case 'R':
    sscanf (optarg, "%f", &gamma_correct_red);
    break;
  case 'G':
    sscanf (optarg, "%f", &gamma_correct_green);
    break;
  case 'B':
    sscanf (optarg, "%f", &gamma_correct_blue);
    break;
  }
    }
  if (verbose)
    printf ("Version %d\n", VERSION);
}
///////////////////////////////////////////////////////////////////////////////////////////////
// Error handling.
///////////////////////////////////////////////////////////////////////////////////////////////
void
HandleError (char *description, int degree)
{
  fprintf (stderr
     ,"An error has occurred.  The description is below...\n");
  fprintf (stderr, "%s\n", description);

  if (degree == FATAL)
    {
      fprintf (stderr, "Program aborting...\n");
      exit (-1);
    }
}
///////////////////////////////////////////////////////////////////////////////////////////////
// RefreshArea.
///////////////////////////////////////////////////////////////////////////////////////////////
static int
AdjustX (int x)
{
  x <<= pix_double;
  x += borderx;
  return x;
}

static int
AdjustY (int y)
{
  y <<= pix_double;
  y += bordery;
  return y;
}

static int
UnAdjustX (int x)
{
  x -= borderx;
  x >>= pix_double;
  return x;
}

static int
UnAdjustY (int y)
{
  y -= bordery;
  y >>= pix_double;
  return y;
}

void
RefreshArea (int x1, int y1, int x2, int y2)  // bounds of refresh area
 {
  BRect rect;
  rect.left = AdjustX (x1);
  rect.top = AdjustY (y1);
  rect.right = AdjustX (x2);
  rect.bottom = AdjustY (y2);

  BView *view = (BView *)BaseWin->view;
  BBitmap *m = (BBitmap *)BaseWin->bitmap;
  view->LockLooper();
  view->DrawBitmap(m, rect, rect);
//  view->Sync();
  view->UnlockLooper();
}
///////////////////////////////////////////////////////////////////////////////////////////////
// Drawing Pixel(s).
///////////////////////////////////////////////////////////////////////////////////////////////
void
Fgl_setpixel(int x, int y, int col)
{
  if (clipping_flag)
    if (x < xclip_x1 || x > xclip_x2 || y < xclip_y1 || y > xclip_y2)
      return;

  BBitmap *m = (BBitmap *)BaseWin->bitmap;
  unsigned char *pixmap = (unsigned char *)m->Bits();
  const int32 lowbyte = m->BytesPerRow();
  BScreen s;
  unsigned char c = s.IndexForColor(pal[col]);

    *(pixmap + (y + bordery) * lowbyte + x + borderx) = c;

}

///////////////////////////////////////////////////////////////////////////////////////////////
// Drawing Lines.
///////////////////////////////////////////////////////////////////////////////////////////////
void
Fgl_hline (int x1, int y1, int x2, int col)
{
  col &= 0xff;
  BView *buffer = (BView *)BaseWin->buffer;
  BView *view = (BView *)BaseWin->view;
    BBitmap *m = (BBitmap *)BaseWin->bitmap;

  buffer->LockLooper();
  view->LockLooper();
  buffer->SetHighColor(pal[col]);

#ifdef ALLOW_PIX_DOUBLING
  if (pix_double)
    buffer->FillRect(RECT (x1 * 2, y1 * 2 ,(x2 - x1) * 2 + 1, 2));
  else
#endif
  buffer->StrokeLine(BPoint(x1 + borderx, y1 + bordery),BPoint(x2 + borderx, y1 + bordery));

  buffer->Sync();
  buffer->UnlockLooper();
  BRect r(x1, y1, x2, y1);
  r.OffsetBy(borderx, bordery);
  view->DrawBitmapAsync(m, r, r);
  view->Sync();
  view->UnlockLooper();  
}

void
Fgl_line (int x1, int y1, int dummy, int y2, int col)
// vertical lines only.
{
  col &= 0xff;
  BView *buffer = (BView *)BaseWin->buffer;
  BView *view = (BView *)BaseWin->view;
  BBitmap *m = (BBitmap *)BaseWin->bitmap;

  buffer->LockLooper();
  view->LockLooper();
  buffer->SetHighColor(pal[col]);

#ifdef ALLOW_PIX_DOUBLING
  if (pix_double)
    buffer->FillRect(RECT(x1 * 2, y1 * 2, 2, (y2 - y1) * 2 + 1));
  else
#endif
  buffer->StrokeLine(BPoint(x1 + borderx, y1 + bordery), BPoint(x1 + borderx, y2 + bordery));

  buffer->Sync();
  buffer->UnlockLooper();

  BRect r(x1, y1, x1, y2);
  r.OffsetBy(borderx, bordery);
  view->DrawBitmapAsync(m, r, r);
  view->Sync();
  view->UnlockLooper();
  
}
///////////////////////////////////////////////////////////////////////////////////////////////
// Drawing Strings.
///////////////////////////////////////////////////////////////////////////////////////////////
void
initfont ()
{
  int i;
  FILE *finf;
  if ((finf = fopen (fontfile, "r")) == 0)
    HandleError ("Can't open the font file", FATAL);
  for (i = 0; i < 256 * 8; i++)
    myfont[i] = fgetc (finf);
  fclose (finf);
}

void
Fgl_setfontcolors (int bg, int fg)
{
  text_fg = fg;
  text_bg = bg;
}

static void
my_x_putchar (const int xx, const int yy, const int c)
{
  BBitmap *m = (BBitmap *)BaseWin->bitmap;
  unsigned char *pixmap = (unsigned char *)m->Bits();
  const int32 lowbyte = m->BytesPerRow();
  BScreen s;
  unsigned char col ;

  const unsigned char bg = s.IndexForColor(pal[text_bg]);
  const unsigned char fg = s.IndexForColor(pal[text_fg]);

  for (int y = 0; y < 8; y++){
    int b = myfont[c * 8 + y];
    for (int x = 0; x < 8; x++){
      if ((b & 0x80) == 0){
        col = bg; 
      }else{
        col = fg; 
      }
            *(pixmap + (yy + y + bordery) * lowbyte + x + xx + borderx) = col;
      b = b << 1;
    }
  }
  
}

void
Fgl_write (const int x, const int y, const char *s)
{
  int i;
  for (i = 0; i < (int) (strlen (s)); i++)
    my_x_putchar (x + i * 8, y, s[i]);
  
  BRect r(x, y, x + 8 * i, y + 8);
  r.OffsetBy(borderx, bordery);

  BView *view = (BView *)BaseWin->view;  
  BBitmap *m = (BBitmap *)BaseWin->bitmap;
  view->LockLooper();
  view->DrawBitmapAsync(m, r, r);
  view->Sync();
  view->UnlockLooper();
}


static void
open_x_putchar (int xx, int yy, int c)
{
  BBitmap *m = (BBitmap *)BaseWin->bitmap;
  unsigned char *pixmap = (unsigned char *)m->Bits();
  const int32 lowbyte = m->BytesPerRow();
  BScreen s;
  unsigned char col ;

  const unsigned char bg = s.IndexForColor(pal[text_bg]);
  const unsigned char fg = s.IndexForColor(pal[text_fg]);

  int x, y, b;
  for (y = 0; y < open_font_height; y++)
    {
      b = open_font[c * open_font_height + y];
      for (x = 0; x < 8; x++)
	{
			if ((b & 0x80) == 0){
				col = bg; 
			}else{
				col = fg; 
			}
            *(pixmap + (yy + y + bordery) * lowbyte + x + xx + borderx) = col;
	  b = b << 1;
	}
    }
}


void
open_write (int x, int y, char *s)
{
int i;
  for (i = 0; i < (int) (strlen (s)); i++)
    open_x_putchar (x + i * 8, y, s[i]);

  BView *view = (BView *)BaseWin->view;	
  view->LockLooper();
  BBitmap *m = (BBitmap *)BaseWin->bitmap;
  BRect r(x, y, x + 8 * i, y + open_font_height);
  r.OffsetBy(borderx, bordery);

  view->DrawBitmapAsync(m, r, r);
  view->Sync();
  view->UnlockLooper();
	
}
///////////////////////////////////////////////////////////////////////////////////////////////
// Drawing Rectangle.
///////////////////////////////////////////////////////////////////////////////////////////////
void
Fgl_fillbox (int x1, int y1, int w, int h, int col)
{
  if (clipping_flag)
    {
      if (x1 < xclip_x1)
  x1 = xclip_x1;
      if (x1 + w > xclip_x2)
  w = xclip_x2 - x1;
      if (y1 < xclip_y1)
  y1 = xclip_y1;
      if (y1 + h > xclip_y2)
  h = xclip_y2 - y1;
    }

  col &= 0xff;
  BView *buffer = (BView *)BaseWin->buffer;
  buffer->LockLooper();
  buffer->SetHighColor(pal[col]);
#ifdef ALLOW_PIX_DOUBLING
  if (pix_double)
    buffer->FillRect(RECT(x1 * 2, y1 * 2, w * 2, h * 2));
  else
#endif
  buffer->FillRect(RECT(x1 + borderx, y1 + bordery, w, h));
  buffer->Sync();
  buffer->UnlockLooper();

  BRect r(x1, y1, x1 + w, y1 + h);
  r.OffsetBy(borderx, bordery);
  BView *view = (BView *)BaseWin->view;
  BBitmap *m = (BBitmap *)BaseWin->bitmap;
  view->LockLooper();
  view->DrawBitmapAsync(m, r, r);
  view->UnlockLooper();
}
///////////////////////////////////////////////////////////////////////////////////////////////
// Rectangle save/restore.
///////////////////////////////////////////////////////////////////////////////////////////////
void
Fgl_getbox (int x1, int y1, int w, int h, void *buf)
{
  unsigned char *b, *p;
  b = (unsigned char *) buf;
  int x, y;
  BBitmap *m = (BBitmap *)BaseWin->bitmap;
  m->Lock();
  unsigned char *pixmap = (unsigned char *)m->Bits();
  const int32 lowbyte = m->BytesPerRow();

  for (y = y1; y < y1 + h; y++){
      p = pixmap + (y + bordery) * lowbyte + x1 + borderx;
    for (x = x1; x < x1 + w; x++){
      *b++ = *p++;
    }
  }
  m->Unlock();
}

void
Fgl_putbox (const int x1, const int y1, const int w, const int h, void *buf)
{
  unsigned char *b = (unsigned char *) buf;
  BBitmap *m = (BBitmap *)BaseWin->bitmap;
  unsigned char *pixmap = (unsigned char *)m->Bits();
  const int32 lowbyte = m->BytesPerRow();

  for (int y = y1; y < y1 + h; y++)
    for (int x = x1; x < x1 + w; x++){
           const unsigned char c = *b++;

      if (clipping_flag)
        if (x < xclip_x1 || x > xclip_x2 || y < xclip_y1 || y > xclip_y2)
          continue;
        *(pixmap + (y + bordery) * lowbyte + x + borderx) = c;
    }

  BRect r(x1, y1, x1 + w, y1 + h);
  r.OffsetBy(borderx, bordery);

  BView *view = (BView *)BaseWin->view;

  view->LockLooper();
  view->DrawBitmapAsync(m, r, r);
  view->UnlockLooper();

}
  

///////////////////////////////////////////////////////////////////////////////////////////////
// Clipping.
///////////////////////////////////////////////////////////////////////////////////////////////
void
Fgl_enableclipping (void)
{
  clipping_flag = 1;
}

void
Fgl_setclippingwindow (int x1, int y1, int x2, int y2)
{
  xclip_x1 = x1;
  xclip_y1 = y1;
  xclip_x2 = x2;
  xclip_y2 = y2;
}

void
Fgl_disableclipping (void)
{
  clipping_flag = 0;
}

///////////////////////////////////////////////////////////////////////////////////////////////
// Event Handling.
///////////////////////////////////////////////////////////////////////////////////////////////
void
HandleEvent (W_Event * event)
{
  switch (event->type)
    {
    case (W_EV_KEY):
      {
  x_key_value = event->key;

  if (x_key_value == B_RETURN)
    x_key_value = 0xd;
  else if (x_key_value == B_BACKSPACE
     || x_key_value == B_DELETE || x_key_value == 8)
    x_key_value = 127;
      }
      break;

    case (W_EV_BUTTON):
      {
  mouse_button = event->button ; ////////////// ev->button;
  if ((modifiers() & B_SHIFT_KEY) != 0)
    cs_mouse_shifted = 1;
  else
    cs_mouse_shifted = 0;
  cs_mouse_handler (mouse_button, 0, 0);
      }
      break;

    case (W_EV_BUTTON_OFF):
      {
  mouse_button = 0;
  cs_mouse_handler (mouse_button, 0, 0);
      }
      break;

     }
}

void
W_NextEvent(W_Event *wevent)
{
    LinCityView *view = (LinCityView *)BaseWin->view;
    view->nextEvent(wevent);
}

int
W_EventsPending()
{
    LinCityView *view = (LinCityView *)BaseWin->view;
    return view->eventsPending();
}

static void
W_GetMouse(int *x, int *y, int *but)
{
    BView *view = (BView *)BaseWin->view;
    BPoint cursor;
    uint32 buttons;
    view->LockLooper();
    view->GetMouse(&cursor, &buttons, false);
    view->UnlockLooper();
    *x = (int)cursor.x;
    *y = (int)cursor.y;
    *but = (buttons == 0) ? 0 : W_LBUTTON;
}

static void
do_call_event (int wait)
{
  int dummy_int, x, y;

  W_Event xev;
  if (W_EventsPending ())  // || wait!=0)

    {
      W_NextEvent (&xev);
      HandleEvent (&xev);
    }
  else if (wait)
    lc_usleep (1000);

  W_GetMouse (&x, &y, &dummy_int);
#ifdef ALLOW_PIX_DOUBLING
  if (pix_double)
    {
      x /= 2;
      y /= 2;
    }
  else
    {
#endif
      x -= borderx;
      y -= bordery;
#ifdef ALLOW_PIX_DOUBLING
    }
#endif
  if (x != cs_mouse_x || y != cs_mouse_y)
    cs_mouse_handler (mouse_button, x - cs_mouse_x, y - cs_mouse_y);
}

void
call_event (void)
{
  do_call_event (0);
}

void
call_wait_event (void)
{
  do_call_event (1);
}
///////////////////////////////////////////////////////////////////////////////////////////////
// Opening image loading.
///////////////////////////////////////////////////////////////////////////////////////////////
void
load_start_image (void)
{
  extern char opening_pic[256];
  BBitmap *openimg =  BTranslationUtils::GetBitmap(opening_pic);

  if (!openimg) return;

  int obx = borderx;
  int oby = bordery;
  borderx = 0;
  bordery = 0;
  Fgl_fillbox (0, 0, 640 + obx * 2, 480 + oby * 2, TEXT_BG_COLOUR);
  borderx = obx;
  bordery = oby;

  BRect r = openimg->Bounds();
  r.OffsetBy(borderx, bordery);

  BView *buffer = (BView *)BaseWin->buffer;
  BView *view = (BView *)BaseWin->view;
  BBitmap *m = (BBitmap *)BaseWin->bitmap;

  buffer->LockLooper();
  view->LockLooper();
  buffer->DrawBitmapAsync(openimg, r);
  buffer->Sync();
  buffer->UnlockLooper();
  delete openimg;

  view->DrawBitmapAsync(m, view->Bounds());
  view->Sync();
  view->UnlockLooper();

  #define SI_BLACK 252
  #define SI_RED 253
  #define SI_GREEN 254
  #define SI_YELLOW 255

  gl_setpalettecolor (SI_BLACK, 0, 0, 0);
  gl_setpalettecolor (SI_RED, 60, 0, 0);
  gl_setpalettecolor (SI_GREEN, 0, 60, 0);
  gl_setpalettecolor (SI_YELLOW, 60, 60, 0);

  start_image_text ();

}

